@@ -1,55 +1,30 @@
-"""A lightweigth MCP server for the BACnet protocol."""
-
 from bacpypes3.app import Application
 from bacpypes3.argparse import SimpleArgumentParser
 from fastmcp import FastMCP
 from fastmcp.server.auth import BearerAuthProvider
 from fastmcp.prompts.prompt import Message
 from fastmcp.resources import ResourceTemplate
-from fastmcp.tools import Tool
-from pydantic import BaseModel
-from pydantic_settings import BaseSettings, SettingsConfigDict
-from typing import Optional
 
-
-class Auth(BaseModel):
-    key: Optional[str] = None
-
-
-class BACnet(BaseModel):
-    host: str = "127.0.0.1"
-    port: int = 47808
-
-
-class Settings(BaseSettings):
-    auth: Auth = Auth()
-    bacnet: BACnet = BACnet()
-    model_config = SettingsConfigDict(
-        env_file=".env", env_file_encoding="utf-8", env_nested_delimiter="__"
-    )
+from bacnet_mcp.settings import Settings
+from bacnet_mcp.utils import get_device
 
 
 settings = Settings()
 
-mcp = FastMCP(
-    name="BACnet MCP Server",
-    auth=(
-        BearerAuthProvider(public_key=settings.auth.key) if settings.auth.key else None
-    ),
-)
-
 
 async def read_property(
-    host: str = settings.bacnet.host,
-    port: int = settings.bacnet.port,
+    name: str | None = None,
+    host: str | None = None,
+    port: int | None = None,
     obj: str = "analogValue",
     instance: str = "1",
     prop: str = "presentValue",
 ) -> str:
     """Reads the content of a BACnet object property on a remote unit."""
-    args = SimpleArgumentParser().parse_args()
+    args = SimpleArgumentParser().parse_args(args=[])
     app = Application().from_args(args)
     try:
+        host, port = get_device(settings, name, host, port)
         res = await app.read_property(f"{host}:{port}", f"{obj},{instance}", f"{prop}")
         return res
     except Exception as e:
@@ -61,42 +36,19 @@ async def read_property(
             app.close()
 
 
-mcp.add_template(
-    ResourceTemplate.from_function(
-        fn=read_property, uri_template="udp://{host}:{port}/{obj}/{instance}/{prop}"
-    )
-)
-
-mcp.add_tool(
-    Tool.from_function(
-        fn=read_property,
-        annotations={
-            "title": "Read Property",
-            "readOnlyHint": True,
-            "openWorldHint": True,
-        },
-    )
-)
-
-
-@mcp.tool(
-    annotations={
-        "title": "Write Property",
-        "readOnlyHint": False,
-        "openWorldHint": True,
-    }
-)
 async def write_property(
-    host: str = settings.bacnet.host,
-    port: int = settings.bacnet.port,
+    name: str | None = None,
+    host: str | None = None,
+    port: int | None = None,
     obj: str = "analogValue,1",
     prop: str = "presentValue",
     data: str = "1.0",
 ) -> str:
     """Writes a BACnet object property on a remote device."""
-    args = SimpleArgumentParser().parse_args()
+    args = SimpleArgumentParser().parse_args(args=[])
     app = Application().from_args(args)
     try:
+        host, port = get_device(settings, name, host, port)
         await app.write_property(f"{host}:{port}", f"{obj}", f"{prop}", f"{data}")
         return f"Write to {obj} {prop} on {host}:{port} has succedeed"
     except Exception as e:
@@ -106,15 +58,12 @@ async def write_property(
             app.close()
 
 
-@mcp.tool(
-    annotations={"title": "Send Who-Is", "readOnlyHint": True, "openWorldHint": True}
-)
 async def who_is(
     low: int,
     high: int,
 ) -> list[str]:
     """Sends a 'who-is' broadcast message."""
-    args = SimpleArgumentParser().parse_args()
+    args = SimpleArgumentParser().parse_args(args=[])
     app = Application().from_args(args)
     try:
         res = await app.who_is(low, high)
@@ -126,16 +75,13 @@ async def who_is(
             app.close()
 
 
-@mcp.tool(
-    annotations={"title": "Send Who-Has", "readOnlyHint": True, "openWorldHint": True}
-)
 async def who_has(
     low: int,
     high: int,
     obj: str,
 ) -> list[str]:
     """Sends a 'who-has' broadcast message."""
-    args = SimpleArgumentParser().parse_args()
+    args = SimpleArgumentParser().parse_args(args=[])
     app = Application().from_args(args)
     try:
         res = await app.who_has(low, high, obj)
@@ -147,7 +93,6 @@ async def who_has(
             app.close()
 
 
-@mcp.prompt(name="bacnet_help", tags={"bacnet", "help"})
 def bacnet_help() -> list[Message]:
     """Provides examples of how to use the BACnet MCP server."""
     return [
@@ -159,7 +104,6 @@ def bacnet_help() -> list[Message]:
     ]
 
 
-@mcp.prompt(name="bacnet_error", tags={"bacnet", "error"})
 def bacnet_error(error: str | None = None) -> list[Message]:
     """Asks the user how to handle an error."""
     return (
@@ -170,3 +114,62 @@ def bacnet_error(error: str | None = None) -> list[Message]:
         if error
         else []
     )
+
+
+class BACnetMCP(FastMCP):
+    def __init__(self, **kwargs):
+        super().__init__(
+            name="BACnet MCP Server",
+            auth=(
+                BearerAuthProvider(public_key=settings.auth.key)
+                if settings.auth.key
+                else None
+            ),
+            **kwargs,
+        )
+
+        self.add_template(
+            ResourceTemplate.from_function(
+                fn=read_property,
+                uri_template="udp://{host}:{port}/{obj}/{instance}/{prop}",
+            )
+        )
+
+        self.tool(
+            read_property,
+            annotations={
+                "title": "Read Property",
+                "readOnlyHint": True,
+                "openWorldHint": True,
+            },
+        )
+
+        self.tool(
+            write_property,
+            annotations={
+                "title": "Write Property",
+                "readOnlyHint": False,
+                "openWorldHint": True,
+            },
+        )
+
+        self.tool(
+            who_is,
+            annotations={
+                "title": "Send Who-Is",
+                "readOnlyHint": True,
+                "openWorldHint": True,
+            },
+        )
+
+        self.tool(
+            who_has,
+            annotations={
+                "title": "Send Who-Has",
+                "readOnlyHint": True,
+                "openWorldHint": True,
+            },
+        )
+
+        self.prompt(bacnet_error, name="bacnet_error", tags={"bacnet", "error"})
+        self.prompt(bacnet_help, name="bacnet_help", tags={"bacnet", "help"})
